%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Example: How to call nscope_daq to record data from the nscope
% NDM 9/19/2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% clear memory and the screen
clc
clear
close all

% what are the inputs to the nscope_daq function? Use the built in help function
help nscope_daq

% set the number of samples to 2000, sample rate to 2kHz, gain of 1 on A and B, function generator to sine wave at 1Hz. Pwm to 5Hz at 25% duty
[dataA dataB dataD1 dataD2 dataD3 dataD4 numDataPoints] = nscope_daq('NumDataPoints',2000,'SampleRate',7,'SensA',1,'SensB',1,'FnType',1,'FnFreq',0,'PwmOn',1,'PwmFreq',3,'PwmDuty',25);

% convert the data to volts
voltA = ((1023 - dataA)/1023-.5)*10;
voltB = ((1023 - dataB)/1023-.5)*10;
digital_d0 = (dataD1)*5;
digital_d1 = (dataD2)*5;
digital_d2 = (dataD3)*5;
digital_d3 = (dataD4)*5;

% convert to time
% Note the divisor! You are responsible for remembering the sample rate
time = (1:numDataPoints)/2000; % divide by 2000 because this data was sampled at 2000Hz

figure;
plot(time,voltA,'r.-',time,voltB,'b.-');axis([0 max(time) -5.5 5.5]);
title('Analog Input');xlabel('Time[s]');ylabel('Volts[V]');legend('A','B');

figure;
plot(time,digital_d0,'g.-',time,digital_d1,'c.-',time,digital_d2,'m.-',time,digital_d3,'y.-');axis([0 max(time) -0.5 5.5]);
title('Digital Input');xlabel('Time[s]');ylabel('Volts[V]');legend('D0','D1','D2','D3');


