function [dataA,dataB,dataD1,dataD2,dataD3,dataD4,gotNumDataPoints] = nscope_daq(varargin)
%Use the nscope as a DAQ
%Options are (case sensitive):
% -'SensA' - 1 for gain of 1
%  and     - 2 for gain of 2, 
%  'SensB' - 3 for gain of 5, 
%          - 4 for gain of 10, 
%          - 5 for gain of 20, 
%          - 6 for gain of 50, 
%          - 7 for gain of 100
%
% -'NumDataPoints' - 1 to something large 
%                  (depends on sample rate)
%                  (max throughput about 20kHz)
%
% -'SampleRate' - 0 for 400kHz, 
%               - 1 for 200kHz, 
%               - 2 for 80kHz, 
%               - 3 for 40kHz, 
%               - 4 for 20kHz, 
%               - 5 for 8kHz, 
%               - 6 for 4kHz, 
%               - 7 for 2kHz, 
%               - 8 for 800Hz, 
%               - 9 for 400Hz,
%               - 10 for 200Hz,
%               - 11 for 80Hz, 
%               - 12 for 40Hz
%
% -'FnType' - 0 for off, 
%           - 1 for Sine
%           - 2 for Triangle, 
%           - 3 for Noise
%
% -'FnFreq' - 0 for 1Hz, 
%           - 1 for 2Hz, 
%           - 2 for 4Hz, 
%           - 3 for 5Hz, 
%           - 4 for 8Hz, 
%           - 5 for 10Hz, 
%           - 6 for 20Hz, 
%           - 7 for 40Hz, 
%           - 8 for 50Hz, 
%           - 9 for 80Hz,
%           - 10 for 100Hz,
%           - 11 for 200Hz, 
%           - 12 for 400Hz, 
%           - 13 for 500Hz, 
%           - 14 for 800Hz, 
%           - 15 for 1kHz, 
%           - 16 for 2kHz, 
%           - 17 for 4kHz, 
%           - 18 for 5kHz
%
% -'Pwm' - 0 for Off
%        - 1 for On
%
% -'PwmFreq' - 0 for 1Hz, 
%           - 1 for 2Hz, 
%           - 2 for 4Hz, 
%           - 3 for 5Hz, 
%           - 4 for 8Hz, 
%           - 5 for 10Hz, 
%           - 6 for 20Hz, 
%           - 7 for 40Hz, 
%           - 8 for 50Hz, 
%           - 9 for 80Hz,
%           - 10 for 100Hz,
%           - 11 for 200Hz, 
%           - 12 for 400Hz, 
%           - 13 for 500Hz, 
%           - 14 for 800Hz, 
%           - 15 for 1kHz, 
%           - 16 for 2kHz, 
%           - 17 for 4kHz, 
%           - 18 for 5kHz
%
% -'PwmDuty' - 0 to 100 for 0% to 100%
% 
%Example: [dataA dataB dataD1 dataD2 dataD3 dataD4] =
%nscope_daq('NumDataPoints',1000,'SampleRate',7); % Sample 1000 points at 2kHz

% defaults, 5V sensitivity, 800 data points, 800Hz sample
% rate, Fn is sine wave at 1Hz, Pwm is on at 1Hz with 50% duty
options = struct('SensA', 1,'SensB',1,'NumDataPoints', 800,'SampleRate', 8,'FnType', 1,'FnFreq',0,'PwmOn',1,'PwmFreq',0,'PwmDuty',50);

% read the acceptable names
optionNames = fieldnames(options);

% count arguments
nArgs = length(varargin);
if round(nArgs/2)~=nArgs/2
   error('EXAMPLE needs propertyName/propertyValue pairs')
end

for pair = reshape(varargin,2,[]) %# pair is {propName;propValue}
   %inpName = lower(pair{1}); %# make case insensitive, NDM-DOESN'T SEEM TO WORK
   inpName = pair{1};

   if any(strmatch(inpName,optionNames))
      % overwrite options. If you want you can test for the right class here
      % Also, if you find out that there is an option you keep getting wrong,
      % you can use "if strcmp(inpName,'problemOption'),testMore,end"-statements
      options.(inpName) = pair{2};
   else
      error('%s is not a recognized parameter name',inpName)
   end
end

% load the lib
javaaddpath(fullfile(pwd,'nu_hidapi-1.1.jar'));
import com.codeminders.hidapi.*;

if ClassPathLibraryLoader.loadNativeHIDLibrary()
    mgr = HIDManager.getInstance();
    devs = mgr.listDevices();
else
    disp('Failed to load HID Library');
    return;
end

% get the nscope from the HID devices
connected = 0;
tried = 0;
while(connected == 0 && tried < 20)
    j = -1;
    for i=1:length(devs)
        if strcmp(devs(i).getProduct_string(),'nScope')
            j = i;
        end
    end
    
    if (j>=0)
        pic = devs(j).open();
        pause(0.01);
        if isempty(pic)
           tried = tried+1; % not really needed anymore, just in case
        else
            pic.enableBlocking();
            connected = 1;
        end 
    end
end

if (connected == 0)
    disp('Fail - could not connect');
    dataA=0;dataB=0;dataD1=0;dataD2=0;dataD3=0;dataD4=0;
    return;
elseif (connected == 1)
    %disp('Connected to nScope');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% send an initialization request
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ComType = 7; % init and request firmware version
PacketID = 0; % didn't get any packets yet
% everything else 0

% build the init packet
output_buffer = zeros(1,65);
% first byte always 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3)); % rightshift is negative, leftshift is positive
% can double check binary values with dec2bin

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
PacketID = 0; % ignore that first PacketID, its bad
% nothing else interesting in this packet

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the function generator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the desired packet
ComType = 1;
output_buffer = zeros(1,65);
% first byte always = 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = 0;
output_buffer(4) = 0;
output_buffer(5) = 0; 
output_buffer(6) = 0; 
output_buffer(7) = bitor(bitor(bitand(31,options.FnFreq),bitshift(options.FnType,6)),bitshift(1,5));
output_buffer(8) = 0; 
output_buffer(9) = 0; 
output_buffer(10) = 0; 

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
% nothing else interesting in this packet

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the PWM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the desired packet
ComType = 1;
output_buffer = zeros(1,65);
% first byte always = 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = 0;
output_buffer(4) = 0;
output_buffer(5) = 0; 
output_buffer(6) = 0; 
output_buffer(7) = 0; 
output_buffer(8) = bitand(options.PwmOn,255);
output_buffer(9) = bitand(options.PwmFreq,255);
output_buffer(10) = bitand(options.PwmDuty,255);

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
% nothing else interesting in this packet

pause(1.5); % give the function generator time to start
%disp('Function generator and Pwm loaded');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the channel sensitivities, sample rate, and start continuous sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the desired packet
ComType = 1;
output_buffer = zeros(1,65);
% first byte always = 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = bitor(bitor(2,bitshift(options.SensA,2)),bitshift(options.SensB,5));
output_buffer(4) = bitand(options.SampleRate,255);
output_buffer(5) = 0; 
output_buffer(6) = 0; 
output_buffer(7) = 0; 
output_buffer(8) = 0; 
output_buffer(9) = 0; 
output_buffer(10) = 0; 

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
% nothing else interesting in this packet

%disp('Collecting...');

% read packets and reply with PacketID until rx the number of desired data points 
if (options.NumDataPoints < 1)
    options.NumDataPoints = 1;
end
gotDataPoints = 0;
while(gotDataPoints < options.NumDataPoints)
    % build a request packet
	output_buffer = zeros(1,65);
	output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
	
	% send the packet
	pic.write(output_buffer);
    
    % read the response
    input_buffer = zeros(1,65);
	input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
	PacketID = bitand(input_buffer(1),7);
	Pwr = bitand(bitshift(input_buffer(1),-3),3);
	SamplesSent = bitor(bitand(bitshift(input_buffer(1),-5),7),bitshift(bitand(input_buffer(2),3),3));
	PowerUsage = bitand(bitshift(input_buffer(2),2),63);
    if SamplesSent == 31 % this means the nscope has updated its properties
        SamplesSent = 0; 
    end
       
	% FrequencyCounter = input_buffer(3 and 4) % don't need
	for(i=1:double(SamplesSent))
        a = uint16(input_buffer(3+i*3)); % -3 on all the index to account for i starting at 1
        b = uint16(input_buffer(2+i*3));
        c = uint16(input_buffer(4+i*3));
        dataA(gotDataPoints+i) = double(bitor(bitshift(bitand(a,3),8),b)); 
        dataB(gotDataPoints+i) = double(bitor(bitshift(a,-2),bitshift(bitand(c,15),6)));
        dataD1(gotDataPoints+i) = double(bitand(bitshift(input_buffer(4+i*3),-4),1));
        dataD2(gotDataPoints+i) = double(bitand(bitshift(input_buffer(4+i*3),-5),1));
        dataD3(gotDataPoints+i) = double(bitand(bitshift(input_buffer(4+i*3),-6),1));
        dataD4(gotDataPoints+i) = double(bitand(bitshift(input_buffer(4+i*3),-7),1));
	end
    
    gotDataPoints = gotDataPoints + double(SamplesSent);
end

%disp('Collection Complete!');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% send packet to stop sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ComType = 1;
Rq = 3; % stop sampling
output_buffer = zeros(1,65);
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = Rq;

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
Pwr = bitand(bitshift(input_buffer(1),-3),3);
SamplesSent = bitor(bitand(bitshift(input_buffer(1),-5),3),bitshift(bitand(input_buffer(2),3),3));
PowerUsage = bitand(bitshift(input_buffer(2),-2),63);
% nothing else interesting in this packet

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% turn off the function generator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build the desired packet
ComType = 1;
output_buffer = zeros(1,65);
% first byte always = 0
output_buffer(2) = bitor(ComType,bitshift(PacketID,3));
output_buffer(3) = 0;
output_buffer(4) = 0;
output_buffer(5) = 0; 
output_buffer(6) = 0; 
output_buffer(7) = 32; % turn off Fn
output_buffer(8) = 0; 
output_buffer(9) = 0; 
output_buffer(10) = 0; 

% send the packet
pic.write(output_buffer);

% read the response
input_buffer = typecast(pic.matlabRead(),'uint8'); % grab an incoming packet
PacketID = bitand(input_buffer(1),7);
Pwr = bitand(bitshift(input_buffer(1),-3),3);
SamplesSent = bitor(bitand(bitshift(input_buffer(1),-5),3),bitshift(bitand(input_buffer(2),3),3));
PowerUsage = bitand(bitshift(input_buffer(2),-2),63);
% nothing else interesting in this packet

%disp('Function generator off');

gotNumDataPoints = gotDataPoints;

% close the port and free the library
pic.close();
mgr.release();

%disp('Disconnected from nScope');


    